########################################
############## DEPRECATED ##############
########################################

@check_run
def sources(config):
    logid = scriptname+'.Collection_sources: '
    ret = list()
    search =  [x[0] for x in keysets_from_dict(config["SOURCE"]) if x[0] != 'last']
    if len(getFromDict(config['SAMPLES'], search)) > 0:
        ret.extend(search)
    log.debug(logid+str(ret))
    return ret

@check_run
def genomepath(s, config):
    logid=scriptname+'.Collection_genomepath: '
    sa = os.path.basename(str(s))
    cond= s.split(os.sep)[-2]
    sk = find_key_for_value(sa, config["SAMPLES"])
    log.debug(logid+'GENOMEPATH: '+str([sa, cond, sk]))
    for skey in sk:
        klist = value_extract(skey, config["SOURCE"])
        for k in klist:
            for x, y in config["GENOME"].items():
                log.debug(logid+'GENOMEPATH: '+str([x, y, k]))
                if str(k) == str(y) or str(k) == str(x):
                    return os.path.join(str(x), str(y))

@check_run
def genome(s, config):
    logid=scriptname+'.Collection_genome: '
    sa = os.path.basename(str(s))
    sp = source_from_sample(str(s), config)
    cond= s.split(os.sep)[-2]
    sk = find_key_for_value(sa, config['SAMPLES'])
    for skey in sk:
        klist = value_extract(skey, config['SOURCE'])
        for k in klist:
            for x, y in config['GENOME'].items():
                log.debug(logid+str([k, x, y]))
                if str(k) == str(x):
                    return str(y)

@check_run
def fullgenomepath(sa, config):
    ret=list()
    for s in sa:
        l = config["GENOME"][s]
        ret.append(os.path.join(str(s), str(l)))
    return ret

@check_run
def genomename(s, config):
    s = os.path.basename(str(s))
    for k, v in config["SAMPLES"].items():
        for g, l in v.items():
            if s in l:
                for x, y in config["GENOME"].items():
                    if g == y:
                        return str(x)

@check_run
def transcriptomepath(s, config):
    logid=scriptname+'.Collection_transcriptomepath: '
    sa = os.path.basename(str(s))
    cond= s.split(os.sep)[-2]
    sk = find_key_for_value(sa, config["SAMPLES"])
    log.debug(logid+'TRANSCRIPTOMEPATH: '+str([sa, cond, sk]))
    for skey in sk:
        klist = value_extract(skey, config["SOURCE"])
        for k in klist:
            for x, y in config["TRANSCRIPTOME"].items():
                log.debug(logid+'TRANSCRIPTOMEPATH: '+str([x, y, k]))
                if str(k) == str(y) or str(k) == str(x):
                    return os.path.join(str(x), str(y))

@check_run
def transcriptome(s, config):
    logid=scriptname+'.Collection_transcriptome: '
    sa = os.path.basename(str(s))
    sp = source_from_sample(str(s), config)
    cond= s.split(os.sep)[-2]
    sk = find_key_for_value(sa, config['SAMPLES'])
    for skey in sk:
        klist = value_extract(skey, config['SOURCE'])
        for k in klist:
            for x, y in config['TRANSCRIPTOME'].items():
                log.debug(logid+str([k, x, y]))
                if str(k) == str(x):
                    return str(y)

@check_run
def fulltranscriptomepath(sa, config):
    ret=list()
    for s in sa:
        l = config["TRANSCRIPTOME"][s]
        ret.append(os.path.join(str(s), str(l)))
    return ret

@check_run
def transcriptomename(s, config):
    s = os.path.basename(str(s))
    for k, v in config["SAMPLES"].items():
        for g, l in v.items():
            if s in l:
                for x, y in config["TRANSCRIPTOME"].items():
                    if g == y:
                        return str(x)


@check_run
def namefromfile(s, config):
    if 'NAME' not in config:
        return ''
    else:
        sa = os.path.basename(str(s))
        cond= s.split(os.sep)[-2]
        sk = find_key_for_value(sa, config["SAMPLES"])
        for skey in sk:
            klist = value_extract(skey, config["NAME"])
            for k in klist:
                if str(skey) == str(cond):
                    return str(k)


@check_run
def namefrompath(p, config):
    p = os.path.dirname(p).split(os.sep)
    klist = getFromDict(config["NAME"], p) if 'NAME' in config else list('')
    for k in klist:
        return str(k)

@check_run
def pathstogenomes(samples, config):
    ret = list()
    for s in samples:
        s = os.path.basename(s)
        for k, v in config["SAMPLES"].items():
            for g, l in v.items():
                if s in l:
                    for x, y in config["GENOME"].items():
                        if g == y:
                            ret.append(os.path.join(str(x), str(y)))
    return sorted(list(set(ret)))


@check_run
def source_from_sample(sample, config):
    logid=scriptname+'.Collection_source_from_sample: '
    s = os.path.dirname(str(sample))
    cond= s.split(os.sep)
    log.debug(logid+str([s, cond]))
    ret = getFromDict(config["SOURCE"], cond)[0]
    return ret


@check_run
def anno_from_file(sample, config, step):
    logid = scriptname+'.Collection_anno_from_file: '
    p = os.path.dirname(genomepath(sample, config))
    s = source_from_sample(sample, config)
    ret = os.path.join(config["REFERENCE"], p, subDict(config["ANNOTATE"], s)[step])
    log.debug(logid+str(ret))
    return ret

@check_run
def anno_from_source(source, config, step):
    logid = scriptname+'.Collection_anno_from_source: '
    s = source.split(os.sep)[0:-1]
    p = s[0]
    samp = source.split(os.sep)[-1]
    log.debug(logid+str(s))
    runstate = runstate_from_sample([samp], config)[0]
    lst = list()
    lst.extend(s)
    lst.append(runstate)
    log.debug(logid+str(lst))
    ret = os.path.join(config["REFERENCE"], p, subDict(config["ANNOTATE"], lst)[step])
    log.debug(logid+str(ret))
    return ret



#! /usr/bin/env python3

import sys
import argparse
import os
import inspect
import gzip
import glob
import re
import logging
import traceback as tb

cmd_subfolder = os.path.join(
    os.path.dirname(
        os.path.realpath(os.path.abspath(inspect.getfile(inspect.currentframe())))
    ),
    "../lib",
)
if cmd_subfolder not in sys.path:
    sys.path.insert(0, cmd_subfolder)

from Logger import *

try:
    scriptname = os.path.basename(inspect.stack()[-1].filename).replace(".py", "")
    log = logging.getLogger(scriptname)

    lvl = log.level if log.level else "INFO"
    for handler in log.handlers[:]:
        handler.close()
        log.removeHandler(handler)

    try:
        handler = logging.FileHandler("LOGS/MONSDA.log", mode="a")
    except:
        handler = logging.FileHandler("log", mode="a")
    handler.setFormatter(
        logging.Formatter(
            fmt="%(asctime)s %(levelname)-8s %(name)-12s %(message)s",
            datefmt="%m-%d %H:%M",
        )
    )
    log.addHandler(handler)
    handler = logging.StreamHandler()
    handler.setFormatter(
        logging.Formatter(
            fmt="%(asctime)s %(levelname)-8s %(name)-12s %(message)s",
            datefmt="%m-%d %H:%M",
        )
    )
    log.addHandler(handler)
    log.setLevel(lvl)

except Exception:
    exc_type, exc_value, exc_tb = sys.exc_info()
    tbe = tb.TracebackException(
        exc_type,
        exc_value,
        exc_tb,
    )
    print("".join(tbe.format()), file=sys.stderr)


def parseargs():
    parser = argparse.ArgumentParser(
        description="Wrapper around snakemake to run config based jobs automatically"
    )
    parser.add_argument(
        "-n",
        "--sample_name",
        action="store_true",
        help=" provide -n if sample names instead of group names should be used for header",
    )
    parser.add_argument(
        "-o",
        "--order",
        action="store_true",
        help="if wanted the order of conditions can be given as comma separated list",
    )
    parser.add_argument(
        "-c", "--conditions", required=True, type=str, help="Conditions to compare"
    )
    parser.add_argument(
        "-t", "--types", required=False, type=str, help="Sequencing types to compare"
    )
    parser.add_argument(
        "-r",
        "--replicates",
        required=True,
        type=str,
        help="Replicates belonging to conditions",
    )
    parser.add_argument(
        "-b", "--batches", required=False, type=str, help="Sample batches to compare"
    )
    parser.add_argument(
        "-p",
        "--paired",
        required=False,
        type=str,
        default=None,
        help="Sequencing strategy for sample name processing",
    )
    parser.add_argument(
        "--table",
        dest="table",
        required=True,
        type=str,
        default="Table",
        help="Name of Table directory to write to",
    )
    parser.add_argument(
        "--anno",
        dest="anno",
        required=True,
        type=str,
        default="Anno.gz",
        help="Name of anno to write to",
    )
    parser.add_argument("--loglevel", default="INFO", help="Log verbosity")
    parser.add_argument("--nextflow", action="store_true", help="Run in nextflow mode")

    if len(sys.argv) == 1:
        parser.print_help(sys.stderr)
        sys.exit(1)

    return parser.parse_args()


class Sample_list(object):

    group_name = ""
    replicate_names = []
    replicate_paths = []
    # the class constructor
    def __init__(self, group_name):
        self.group_name = group_name
        self.replicate_names = list()
        self.replicate_paths = list()
        self.replicate_types = list()
        self.replicate_batches = list()


def prepare_table(
    conditions,
    replicates,
    types,
    batches,
    paired,
    table,
    anno,
    sample_name=None,
    order=None,
    nextflow=None,
):
    try:
        # slist,
        logid = scriptname + ".prepare_table: "
        my_groups = {}
        list_size = 0

        # CLEANUP
        # oldtab = os.path.abspath(table)
        oldanno = os.path.abspath(anno)
        # for oldfile in glob.glob(oldtab):
        #    os.rename(oldfile,oldfile+'.bak')
        #    log.warning(logid+'Found old table file'+oldfile+', was moved to '+oldfile+'.bak')
        for oldfile in glob.glob(oldanno):
            os.rename(oldfile, oldfile + ".bak")
            log.warning(
                logid
                + "Found old anno file"
                + oldfile
                + ", was moved to "
                + oldfile
                + ".bak"
            )

        replist = str(replicates).strip().split(",")
        condlist = str(conditions).strip().split(",")
        typelist = str(types).strip().split(",") if types is not None else None
        batchlist = str(batches).strip().split(",") if batches is not None else None
        pairedlist = str(paired).strip().split(",") if paired is not None else None

        log.debug(logid + "REPS: " + str(replist) + "\tLEN: " + str(len(replist)))
        log.debug(logid + "CONDS: " + str(condlist) + "\tLEN: " + str(len(condlist)))

        if types is not None:
            log.debug(
                logid + "TYPES: " + str(typelist) + "\tLEN: " + str(len(typelist))
            )

        if batches is not None:
            log.debug(
                logid + "BATCHES: " + str(batchlist) + "\tLEN: " + str(len(batchlist))
            )

        if paired is not None:
            log.debug(
                logid + "PAIRED: " + str(pairedlist) + "\tLEN: " + str(len(pairedlist))
            )

        for i in range(len(replist)):
            rep = None
            cond = None
            typ = None
            bat = None

            rep = str(replist[i]) if not nextflow else str(os.path.basename(replist[i]))
            cond = str(condlist[i])
            typ = str(typelist[i]) if types is not None else None
            bat = str(batchlist[i]) if batches is not None else None

            if not rep or not cond:
                log.warning(logid + "No rep/cond found for sample " + str(replist[i]))

            log.debug(logid + "rep/cond: " + str([rep, cond]))

            list_size += 1

            if cond in my_groups:
                my_groups[cond].replicate_paths.append(rep)
                my_groups[cond].replicate_names.append(str.split(os.sep, rep)[-1])
                if typ is not None:
                    my_groups[cond].replicate_types.append(typ)
                if bat is not None:
                    my_groups[cond].replicate_batches.append(bat)
            else:
                my_groups[cond] = make_sample_list(cond)
                my_groups[cond].replicate_paths.append(rep)
                my_groups[cond].replicate_names.append(str.split(os.sep, rep)[-1])
                if typ is not None:
                    my_groups[cond].replicate_types.append(typ)
                if bat is not None:
                    my_groups[cond].replicate_batches.append(bat)

        log.debug(logid + "MyGroups: " + str(my_groups.keys()))

        myMatrix = []
        myMatrix.append([])
        myMatrix[0].append("")
        sample_counter = 0

        conds = []
        if order:
            conds = order.split(",")
        else:
            conds = [x for x in my_groups.keys()]

        log.debug(logid + "CONDS: " + str(conds))
        typeanno = list()
        batchanno = list()
        for gruppies in conds:
            condition_index = -1
            rep_nr = 0
            for replicates in my_groups[gruppies].replicate_paths:
                log.info(logid + "Processing: " + str(replicates))
                condition_index += 1
                sample_counter += 1
                rep_nr += 1

                if sample_name:
                    myMatrix[0].append(
                        my_groups[gruppies].replicate_names[condition_index]
                    )
                    os.symlink(
                        os.path.abspath(replicates),
                        replicates.replace(
                            "quant.sf",
                            my_groups[gruppies].replicate_names[condition_index]
                            + ".sf",
                        ),
                    )
                    typeanno.append(
                        my_groups[gruppies].replicate_types[condition_index]
                    )
                    batchanno.append(
                        my_groups[gruppies].replicate_batches[condition_index]
                    )
                else:
                    myMatrix[0].append(
                        str(my_groups[gruppies].group_name) + "_" + str(rep_nr)
                    )
                    os.symlink(
                        os.path.abspath(replicates),
                        os.path.join(
                            os.path.abspath(table),
                            str(my_groups[gruppies].group_name)
                            + "_"
                            + str(rep_nr)
                            + ".sf",
                        ),
                    )
                    typeanno.append(
                        my_groups[gruppies].replicate_types[condition_index]
                    )
                    batchanno.append(
                        my_groups[gruppies].replicate_batches[condition_index]
                    )

        line = "\t".join(myMatrix[0])
        annos = list()

        for i in range(1, len(myMatrix[0])):
            c = myMatrix[0][i]
            a = str.join("_", str(c).split("_")[:-1])
            a += "\t" + str(typeanno[i - 1]) if types is not None else None
            a += "\t" + str(batchanno[i - 1]) if batches is not None else None
            annos.append(str(c) + "\t" + str(a))

        with gzip.open(anno, "wb") as a:
            a.write(bytes("\n".join(annos) + "\n", encoding="UTF-8"))

    except Exception as err:
        exc_type, exc_value, exc_tb = sys.exc_info()
        tbe = tb.TracebackException(
            exc_type,
            exc_value,
            exc_tb,
        )
        log.error(logid + "".join(tbe.format()))


def make_sample_list(group_name):
    sample_list = Sample_list(group_name)
    return sample_list


####################
####    MAIN    ####
####################

if __name__ == "__main__":

    logid = scriptname + ".main: "
    try:
        args = parseargs()
        makelogdir("LOGS")
        try:
            log = setup_logger(
                name=scriptname,
                log_file="stderr",
                logformat="%(asctime)s %(name)-12s %(levelname)-8s %(message)s",
                datefmt="%m-%d %H:%M",
                level=args.loglevel,
            )
            # log.addHandler(logging.StreamHandler(sys.stderr))  # streamlog
        except:
            log = logging.getLogger(os.path.basename(inspect.stack()[-1].filename))

        prepare_table(
            args.conditions,
            args.replicates,
            args.types,
            args.batches,
            args.paired,
            args.table,
            args.anno,
            args.sample_name,
            args.order,
        )
    except Exception as err:
        exc_type, exc_value, exc_tb = sys.exc_info()
        tbe = tb.TracebackException(
            exc_type,
            exc_value,
            exc_tb,
        )
        log.error(logid + "".join(tbe.format()))

#
# build_salmon_table.py ends here



# CollectBamStat.py ---
#
# Filename: CollectBamStat.py
# Description:
# Author: Joerg Fallmann
# Maintainer:
# Created: Tue May 15 12:44:06 2018 (+0200)
# Version:
# Package-Requires: ()
# Last-Updated: Wed Jun 20 11:44:19 2018 (+0200)
#           By: Joerg Fallmann
#     Update #: 320
# URL:
# Doc URL:
# Keywords:
# Compatibility:
#
#

# Commentary:
#
#
#
#

# Change Log:
#
#
#
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.
#
#

# Code:

### IMPORTS
import argparse
import pprint  # pp = pprint.PrettyPrinter(indent=4) #pp.pprint(stuff)
from io import StringIO
import os
import gzip
import re
import sys
import pysam
from pyfaidx import Fasta

# import importlib
import multiprocessing
from multiprocessing import Manager
import traceback as tb
import inspect

# Container
import collections

# logging
import logging

####load own modules
cmd_subfolder = os.path.join(
    os.path.dirname(
        os.path.realpath(os.path.abspath(inspect.getfile(inspect.currentframe())))
    ),
    "../lib",
)
if cmd_subfolder not in sys.path:
    sys.path.insert(0, cmd_subfolder)

from Logger import *

### MAIN

try:
    scriptname = os.path.basename(inspect.stack()[-1].filename).replace(".py", "")
    log = logging.getLogger(scriptname)

    lvl = log.level if log.level else "INFO"
    for handler in log.handlers[:]:
        handler.close()
        log.removeHandler(handler)

    handler = logging.FileHandler("LOGS/MONSDA.log", mode="a")
    handler.setFormatter(
        logging.Formatter(
            fmt="%(asctime)s %(levelname)-8s %(name)-12s %(message)s",
            datefmt="%m-%d %H:%M",
        )
    )
    log.addHandler(handler)
    handler = logging.StreamHandler()
    handler.setFormatter(
        logging.Formatter(
            fmt="%(asctime)s %(levelname)-8s %(name)-12s %(message)s",
            datefmt="%m-%d %H:%M",
        )
    )
    log.addHandler(handler)
    log.setLevel(lvl)

except Exception:
    exc_type, exc_value, exc_tb = sys.exc_info()
    tbe = tb.TracebackException(
        exc_type,
        exc_value,
        exc_tb,
    )
    print("".join(tbe.format()), file=sys.stderr)


def parseargs():
    parser = argparse.ArgumentParser(
        description="Read BAM file read by read and collect mapping statistics"
    )
    parser.add_argument(
        "-f", "--fasta", type=str, help="Reference genome FASTA, needs to be indexed"
    )
    parser.add_argument(
        "-b",
        "--bams",
        type=str,
        help="Mapped reads BAM(s) comma separated, need to besorted and indexed",
    )
    parser.add_argument(
        "-o", "--outdir", type=str, default=".", help="Output directory"
    )
    parser.add_argument(
        "-z",
        "--procs",
        type=int,
        default=1,
        help="Number of parallel processed to run this job with",
    )
    parser.add_argument(
        "-v",
        "--verbosity",
        type=int,
        default=0,
        choices=[0, 1],
        help="increase output verbosity",
    )

    return parser.parse_args()


def collectstats(fasta, bams, outdir, procs, verbosity):

    if outdir:
        print("Checking or creating outdir " + str(outdir))
        if not os.path.isabs(outdir):
            outdir = os.path.abspath(outdir)
        if not os.path.exists(outdir):
            os.makedirs(outdir)
    else:
        outdir = os.path.abspath(os.getcwd())

    ###Multithreading
    num_processes = procs or 1
    pool = multiprocessing.Pool(processes=num_processes)
    # 	result_list = []

    for bam in bams.split(","):
        fn = os.path.basename(bam)
        out = os.path.join(outdir, fn + ".stats.gz")

        if os.path.isfile(out):
            print("File " + out + " exists, will be deleted")
            os.remove(out)
        ####Check if index files available
        # 		try:
        # 			check_idx(fasta)
        # 		except:
        # 			print('No index for file: '+fasta+'. Please create!')
        # 		readnum = 0
        write_header(out)
        samfile = parse_bam(bam)
        header = ""

        try:
            samfile.check_index()
        except:
            print("No index for file: " + bam + ". Please create!")
        try:
            header = samfile.header["SQ"]

        except:
            print(
                "Could not read header of file "
                + bam
                + ". Multithreading only per file not per chromosome."
            )

        if "SQ" in header:
            chroms = []
            for k, v in header["SQ"].items():
                print(k, v)
                chroms.extend = str.split(":", k)[1]
            for chrom in chroms:
                pool.apply_async(
                    collect, args=(bam, fasta, out, chrom)
                )  # , callback = log_result)
        else:
            # 			res = pool.apply_async(test,args=(27,))
            res = pool.apply_async(
                collect,
                args=(
                    bam,
                    fasta,
                    out,
                ),
            )  # , callback = log_result)
    # 			collect(samfile, fasta, out)

    pool.close()
    pool.join()


# 	print(res.get())


def test(x):
    try:
        with open("bla", "a") as h:
            print(x ** 2, file=h)
    # 		raise Exception
    # 		return a
    except Exception as error:
        # capture the exception and bundle the traceback
        # in a string, then raise new exception with the string traceback
        raise Exception("".join(tb.format_exception(*sys.exc_info())))


def collect(bam, fasta, out, chrom=None):

    # 	with open('bla','a') as h:
    # 		print('Collected', file=h)

    statistics = collections.OrderedDict()
    statistics["reads"] = collections.OrderedDict()
    statistics["ref"] = collections.OrderedDict()

    samfile = parse_bam(bam)

    if chrom is not "None":
        for read in samfile.fetch(chrom):
            if not read.is_unmapped:
                chrom = read.reference_name
                cigar = read.cigarstring
                if "*" in cigar:
                    continue
                seq = read.query_alignment_sequence
                alignmentstart = read.reference_start

                if (len(statistics["reads"]) > 0) and (
                    chrom not in statistics["reads"]
                ):  # or (alignmentstart not in statistics['reads'][chrom])):
                    print("Fetching reference sequence and printing")

                    write_stats(out, statistics, fasta)
                    statistics["reads"].clear()
                    statistics["ref"].clear()

                if chrom not in statistics["reads"]:
                    print("Parsing Chromosome " + chrom)

                get_stats(chrom, cigar, seq, alignmentstart, statistics)

    else:
        for read in samfile.fetch():
            if not read.is_unmapped:
                chrom = read.reference_name
                cigar = read.cigarstring
                if "*" in cigar:
                    continue
                seq = read.query_alignment_sequence
                alignmentstart = read.reference_start

                if (len(statistics["reads"]) > 0) and (
                    chrom not in statistics["reads"]
                ):  # or (alignmentstart not in statistics['reads'][chrom])):
                    print("Fetching reference sequence and printing")

                    write_stats(out, statistics, fasta)
                    statistics["reads"].clear()
                    statistics["ref"].clear()

                if chrom not in statistics["reads"]:
                    print("Parsing Chromosome " + chrom)

                get_stats(chrom, cigar, seq, alignmentstart, statistics)

    write_stats(out, statistics, fasta)
    close_bam(samfile)


# 	print('Reads parsed: ' + str(readnum))


def parse_bam(bam):
    if ".bam" in bam:
        return pysam.AlignmentFile(bam, "rb")
    elif ".sam" in bam:
        return pysam.AlignmentFile(bam, "r")


def read_head(bam):
    if ".bam" in bam:
        return pysam.AlignmentFile(bam, "rb").header
    elif ".sam" in bam:
        return pysam.AlignmentFile(bam, "r").header


def write_bam(bam, template):
    return pysam.AlignmentFile(bam, "wh", template=template)


def close_bam(samfile):
    samfile.close()


def check_idx(file):
    if (
        (os.path.isfile(file + ".idx"))
        or (os.path.isfile(str.join(".", str.split(".", file)[0:-1], "fai")))
        or (os.path.isfile(str.join(".", str.split(".", file)[0:-1], "faidx")))
    ):
        return True
    else:
        return False


def get_stats(chrom, cigar, seq, alignmentstart, statistics):

    # 	with open('bla','a') as h:
    # 		print('Fetching Stats', file=h)

    pattern = re.compile("([0-9]*)([DMINSHP=XB])")
    pos = alignmentstart + 1
    char = 0
    if chrom not in statistics["reads"]:
        statistics["reads"][chrom] = collections.OrderedDict()

    for n, c in pattern.findall(cigar):
        if n:
            n = int(n)
        else:
            n = 1

        if c == "M" or c == "X" or c == "=":
            for i in range(n):
                if pos not in statistics["reads"][chrom]:
                    statistics["reads"][chrom][pos] = collections.OrderedDict()
                if seq[char] not in statistics["reads"][chrom][pos]:
                    statistics["reads"][chrom][pos][seq[char]] = 0

                statistics["reads"][chrom][pos][seq[char]] += 1
                char += 1
                pos += 1
        if c in ["D", "N"]:
            pos += n
        if c in ["I"]:
            char += n
        if c in ["H", "P", "B", "S"]:  # No idea what PB mean
            continue


# 	biggestseen = pos


def get_ref(fa, statistics):
    faseq = Fasta(fa)

    # 	with open('bla','a') as h:
    # 		print('Fetching Ref', file=h)

    for chrom in statistics["reads"]:
        if chrom not in statistics["ref"]:
            statistics["ref"][chrom] = collections.OrderedDict()
        for pos in statistics["reads"][chrom]:
            if pos not in statistics["ref"][chrom]:
                statistics["ref"][chrom][pos] = collections.OrderedDict()
            nuc = faseq[chrom][pos - 1].seq.upper()
            statistics["ref"][chrom][pos] = nuc


def write_header(bam):
    head = (
        str.join(
            "\t",
            ["Chromosome", "Position", "Ref", "A", "C", "G", "T", "N", "RelRefFreq"],
        )
        + "\n"
    )
    o = gzip.open(bam, "ab")
    o.write(bytes(head, encoding="UTF-8"))
    o.close()


def write_stats(bam, statistics, fasta, start=None):

    o = gzip.open(bam, "ab")
    outstr = ""
    coverage = ""

    get_ref(fasta, statistics)

    for chrom in statistics["reads"]:
        for pos in statistics["reads"][chrom]:
            if start is not None:
                if int(pos) > int(start):
                    continue
            outstr = (
                str.join("\t", [str(chrom), str(pos), statistics["ref"][chrom][pos]])
                + "\t"
            )
            sum = 0
            for nuc in ["A", "C", "G", "T", "N"]:
                if nuc in statistics["reads"][chrom][pos]:
                    sum += statistics["reads"][chrom][pos][nuc]
                    coverage += str(statistics["reads"][chrom][pos][nuc]) + "\t"
                else:
                    coverage += str(0) + "\t"
            if statistics["ref"][chrom][pos] in statistics["reads"][chrom][pos]:
                coverage += str(
                    statistics["reads"][chrom][pos][statistics["ref"][chrom][pos]] / sum
                )
            else:
                coverage += str(0)
            outstr += coverage + "\n"
            o.write(bytes(outstr, encoding="UTF-8"))
            outstr = ""
            coverage = ""
    ###Must not change size during iteration
    # 			del statistics['reads'][chrom][pos]
    # 			del statistics['ref'][chrom][pos]
    o.close()


if __name__ == "__main__":
    args = parseargs()
    collectstats(args.fasta, args.bams, args.outdir, args.procs, args.verbosity)

#
# CollectBamStat.py ends here
